==> trend_etl-1.4.4.tar.gz: trend_etl-1.4.4/trend_etl/transform/groups.py <==
from pyspark.sql import DataFrame
from typing import List

# from pyspark.sql.column import Column
from pyspark.sql.functions import col, lit, named_struct, get_json_object
from pyspark.sql.types import StringType, BinaryType

import trend_etl.schema.fields as fields
from trend_etl.functions.index import (
    jobid_2_timestamp,
    parse_facebook_groups,
    conv_timestamp,
    get_collected_day,
)


# arguments:
#   archive_df: the dataframe from the archives
#   group_ids: the group ids used in filtering
#   include_flag: whether to use the filtering logic to include/exclude
#   provenance: short description of the origin (kafka, s3 archive, etc.)
def from_archive_get_bronze_groups_df(
    archive_df: DataFrame, group_ids: List[str], include_flag: bool
) -> DataFrame:
    field = col(fields.archive_value_field)

    return (
        archive_df.select(
            get_json_object(field, "$.id").isin(group_ids).alias("_filter"),
            col(fields.archive_value_field)
            .cast(BinaryType())
            .alias(fields.bronze_value_field),
            lit("archive/facebook/groups").alias(fields.bronze_provenance_field),
            jobid_2_timestamp(fields.archive_job_id_field).alias(
                fields.bronze_timestamp_field
            ),
        )
        .where(col("_filter") == include_flag)
        .drop("_filter")
        .withColumn(
            fields.collected_day_field,
            get_collected_day(col(fields.bronze_timestamp_field)),
        )
    )


# arguments:
#   input_df: dataframe received from reading a kafka topic
#   target_topic: source topic from kafka
# return: a dataframe in the schema expected for bronze tables
def from_kafka_get_bronze_groups_df(
    input_df: DataFrame, target_topic: str
) -> DataFrame:
    return input_df.select(
        col(fields.kafka_value_field).alias(fields.bronze_value_field),
        lit(f"kafka/{target_topic}").alias(fields.bronze_provenance_field),
        col(fields.kafka_timestamp_field).alias(fields.bronze_timestamp_field),
        get_collected_day(col(fields.kafka_timestamp_field)).alias(
            fields.collected_day_field
        ),
    )


# arguments:
#   streaming_bronze: streaming dataframe from bronze table
#   group_idents: groups uuid idents table
# return: dataframe in the schema of facebook_groups
def from_bronze_get_groups_df(
    streaming_bronze: DataFrame,
    group_idents: DataFrame,
) -> DataFrame:
    parsed_df = streaming_bronze.select(
        parse_facebook_groups(col(fields.bronze_value_field).cast(StringType())).alias(
            "value"
        ),
        col(fields.bronze_timestamp_field),
        col(fields.collected_day_field),
    )
    flattened_df = parsed_df.select(
        col("value.*"),
        col(fields.bronze_timestamp_field),
        col(fields.collected_day_field),
    )

    # NOTE: left outer join (below) will cause any nulls to be
    #       detected as a downstream error.

    # join with group_idents
    group_idents_df = flattened_df.join(
        group_idents,
        fields.idents_source_id_field,
        "left_outer",
    )

    groups_df = group_idents_df.select(
        col(fields.idents_group_id_field),
        col("email").alias(fields.facebook_groups_email),
        col("description").alias(fields.facebook_groups_description),
        conv_timestamp("created_time").alias(fields.facebook_groups_created_at),
        col("name").alias(fields.facebook_groups_name),
        col("privacy").alias(fields.facebook_groups_privacy),
        col("member_count").alias(fields.facebook_groups_member_count),
        named_struct(lit("id"), col("id")).alias(fields.facebook_groups_source_keys),
        col(fields.bronze_timestamp_field),
        col(fields.collected_day_field),
    )

    return groups_df

==> trend_etl-1.4.7.tar.gz: trend_etl-1.4.7/trend_etl/transform/groups.py <==
from pyspark.sql import DataFrame
from typing import List

# from pyspark.sql.column import Column
from pyspark.sql.functions import col, lit, struct, get_json_object
from pyspark.sql.types import StringType, BinaryType

import trend_etl.schema.fields as fields
from trend_etl.functions.index import (
    jobid_2_timestamp,
    parse_facebook_groups,
    conv_timestamp,
    get_collected_day,
)


# arguments:
#   archive_df: the dataframe from the archives
#   group_ids: the group ids used in filtering
#   include_flag: whether to use the filtering logic to include/exclude
#   provenance: short description of the origin (kafka, s3 archive, etc.)
def from_archive_get_bronze_groups_df(
    archive_df: DataFrame, group_ids: List[str], include_flag: bool
) -> DataFrame:
    field = col(fields.archive_value_field)

    return (
        archive_df.select(
            get_json_object(field, "$.id").isin(group_ids).alias("_filter"),
            col(fields.archive_value_field)
            .cast(BinaryType())
            .alias(fields.bronze_value_field),
            lit("archive/facebook/groups").alias(fields.bronze_provenance_field),
            jobid_2_timestamp(fields.archive_job_id_field).alias(
                fields.bronze_timestamp_field
            ),
        )
        .where(col("_filter") == include_flag)
        .drop("_filter")
        .withColumn(
            fields.collected_day_field,
            get_collected_day(col(fields.bronze_timestamp_field)),
        )
    )


# arguments:
#   input_df: dataframe received from reading a kafka topic
#   target_topic: source topic from kafka
# return: a dataframe in the schema expected for bronze tables
def from_kafka_get_bronze_groups_df(
    input_df: DataFrame, target_topic: str
) -> DataFrame:
    return input_df.select(
        col(fields.kafka_value_field).alias(fields.bronze_value_field),
        lit(f"kafka/{target_topic}").alias(fields.bronze_provenance_field),
        col(fields.kafka_timestamp_field).alias(fields.bronze_timestamp_field),
        get_collected_day(col(fields.kafka_timestamp_field)).alias(
            fields.collected_day_field
        ),
    )


# arguments:
#   streaming_bronze: streaming dataframe from bronze table
#   group_idents: groups uuid idents table
# return: dataframe in the schema of facebook_groups
def from_bronze_get_groups_df(
    streaming_bronze: DataFrame,
    group_idents: DataFrame,
) -> DataFrame:
    parsed_df = streaming_bronze.select(
        parse_facebook_groups(col(fields.bronze_value_field).cast(StringType())).alias(
            "value"
        ),
        col(fields.bronze_timestamp_field),
        col(fields.collected_day_field),
    )
    flattened_df = parsed_df.select(
        col("value.*"),
        col(fields.bronze_timestamp_field),
        col(fields.collected_day_field),
    )

    # NOTE: left outer join (below) will cause any nulls to be
    #       detected as a downstream error.

    # join with group_idents
    group_idents_df = flattened_df.join(
        group_idents,
        fields.idents_source_id_field,
        "left_outer",
    )

    groups_df = group_idents_df.select(
        col(fields.idents_group_id_field),
        col("email").alias(fields.facebook_groups_email),
        col("description").alias(fields.facebook_groups_description),
        conv_timestamp("created_time").alias(fields.facebook_groups_created_at),
        col("name").alias(fields.facebook_groups_name),
        col("privacy").alias(fields.facebook_groups_privacy),
        col("member_count").alias(fields.facebook_groups_member_count),
        struct(col("id").alias("id")).alias(fields.facebook_groups_source_keys),
        col(fields.bronze_timestamp_field),
        col(fields.collected_day_field),
    )

    return groups_df
